/**
 * ModServ IRC Services
 * Version 2009
 * Copyright (C) 2007, 2008 Christian Jurk <commx@commx.ws>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __MS_CHANNELS_H
#define __MS_CHANNELS_H

#include <QtGlobal>
#include <QString>
#include <QHash>
#include <QDateTime>
#include <QList>
#include <MS_Users.h>

class MChanBan {
  private:
    QString m_banMask;
    QString m_banBy;
    
  public:
    MChanBan(const QString &strBanmask, const QString &strBannedby = QString(""));
    const QString &banmask(void);
    const QString &bannedBy(void);
};

class MChanMember {
  private:
    MUser *m_user;
    quint32 m_flags;

  public:
    MChanMember(MUser *user, const quint32 &flags = 0);
    void setFlags(const quint32 &flags);
    const quint32 &flags(void);
    MUser *user(void);
    
    /* these items have to be declared as int instead of quint32,
       as MinGW and MSVC seem to understand quint32 as non-primitive. */
    const static int Flag_Ban = 0x1;
    const static int Flag_ChanOp = 0x2;
    const static int Flag_Voice = 0x4;
};

class MChannel {
  private:
    QString m_name;
    QString m_key;
    QString m_topic;
    QHash<unsigned int, MChanMember *> m_memberList;
    QList<MChanBan *> m_banList;
    QDateTime m_creationTime;
    quint32 m_modeflags;
    quint32 m_limit;
    
  public:
    MChannel(const QString &strName);
    ~MChannel();
    void setName(const QString &strName);
    void setCreationTS(const QDateTime &t_src);
    void setModeflags(const quint32 &flags);
    void setKey(const QString &strKey);
    void setLimit(const quint32 &limit);
    void setTopic(const QString &strTopic);
    const QString &name(void);
    const QDateTime &creationTS(void);
    const quint32 &modeflags(void);
    const QString &key(void);
    const quint32 &limit(void);
    const QString &topic(void);
    void addMember(MUser *user, const quint32 &flags);
    void removeMember(MUser *user);
    MChanMember *member(MUser *user);
    const QHash<unsigned int, MChanMember *> &memberList(void);
    void clearBanlist(void);
    void addBan(const QString &strBanMask, const QString &strBanBy = QString(""));
    MChanBan *findBan(const QString &strBanMask);
    void removeBan(const QString &strBanMask);
    int memberCount(void);
    int banCount(void);
    void sendOp(MUser *user, const QString &strSender);
    void sendVoice(MUser *user, const QString &strSender);
    void sendKickBan(MUser *user, const QString &strSender,
      const QString &strBannedBy = QString(""), const QString &strBanReason = QString(""));
    
    static void init(void);
    static MChannel *find(const QString &strName);

    const static int Flag_NoCtcp = 0x00001; /* flag C */
    const static int Flag_NoControlCodes = 0x00002; /* flag c */
    const static int Flag_DelayedJoin = 0x00004; /* flag D */
    const static int Flag_HiddenUsers = 0x00008; /* flag d */
    const static int Flag_InviteOnly = 0x00010; /* flag i */
    const static int Flag_Key = 0x00020; /* flag k */
    const static int Flag_Limit = 0x00040; /* flag l */
    const static int Flag_AuthedMsgOnly = 0x00080; /* flag M */
    const static int Flag_Moderated = 0x00100; /* flag m */
    const static int Flag_NoNotice = 0x00200; /* flag N */
    const static int Flag_NoExternalMsg = 0x00400; /* flag n */
    const static int Flag_Private = 0x00800; /* flag p */
    const static int Flag_RegOnly = 0x01000; /* flag r */
    const static int Flag_StripCodes = 0x02000; /* flag S */
    const static int Flag_Secure = 0x04000; /* flag s */
    const static int Flag_NoMultiTarget = 0x08000; /* flag T */
    const static int Flag_OnlyOpTopic = 0x10000; /* flag t */
    const static int Flag_HideQuitPart = 0x20000; /* flag u */
    const static int Flag_SSL = 0x40000; /* flag z */
};

//extern MChannel *findChannel(const QString &strName);

extern QHash<QString, MChannel *> channelList;
extern QHash<char, quint32> cmodeList;

#endif /* __MS_CHANNELS_H */
