/**
 * ModServ IRC Services
 * Version 2009
 * Copyright (C) 2007, 2008 Christian Jurk <commx@commx.ws>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __MS_NETWORKING_H
#define __MS_NETWORKING_H

#include <QString>
#include <QTcpSocket>

#define BUFSIZE 512 /* IRC RFC buffer size, 512 bytes per line max */
#define SOCKRETRIES 10 /* number of retries before stop to try to connect */

class MTcpSocket : public QTcpSocket {
  Q_OBJECT
  
  private:
    double m_bytesRx; /* number of bytes read */
    double m_bytesTx; /* number of bytes written */
    quint16 m_conAttempts;
    quint16 m_remotePort;
    QString m_remoteHost;

  public:
    MTcpSocket(QString strHost = QString(""), quint16 nPort = 0);
    ~MTcpSocket();
    void connectTo(const QString &strHost, quint16 nPort);
    void scheduleRetry(void);
    void send(const QString &strBuffer, bool appendNl = true);
    const double &getBytesRx(void);
    const double &getBytesTx(void);
    
  public slots:
    void onSocketConnected();
    void onSocketDisconnect();
    void onSocketError(QAbstractSocket::SocketError sockErr);
    void onSocketRead();
    void reconnect(void);
    
  signals:
    void readyToParse(const QString &strBuffer);
    void readyToAuth();
};

#endif /* __MS_NETWORKING_H */
