/**
 * ModServ IRC Services
 * Version 2009
 * Copyright (C) 2007, 2008 Christian Jurk <commx@commx.ws>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __MS_SERVERS_H
#define __MS_SERVERS_H

#include <QString>
#include <QHash>
#include <QDateTime>

/* default server lengths */
#define L_SRV_NICKLEN     20
#define L_SRV_USERLEN     10
#define L_SRV_ACCOUNTLEN  20
#define L_SRV_TOPICLEN    300
#define L_SRV_AWAYLEN     260

class MUser;

class MServer {
  private:
    QString m_serverName;
    QString m_serverDescription;
    unsigned int m_numeric;
    unsigned short m_flags;
    char m_state;
    QDateTime m_timeBooted;
    QDateTime m_timeLinked;
    
  public:
    MServer();
    void setServerName(const QString &strName);
    void setServerDescription(const QString &strDesc);
    void setServerNumeric(unsigned int num);
    void setServerFlags(unsigned short flags);
    void setServerState(char state);
    void setServerTSBooted(const QDateTime &timestamp);
    void setServerTSLinked(const QDateTime &timestamp);
    
    const QString &name(void);
    const QString &description(void);
    unsigned int numeric(void);
    const QString numeric64(void);
    unsigned short flags(void);
    char state(void);
    const QDateTime &bootTS(void);
    const QDateTime &linkTS(void);
    
    QList<MUser *> userList;
    
    const static int STATE_CONNECTED = 1;
    const static int STATE_END_OF_BURST = 2;
    const static int STATE_LINKED = 3;
    
    const static int FL_HIDDEN = 0x10;
    const static int FL_SERVICES = 0x20;
};

extern MServer *findServerByName(const QString &strName);
extern MServer *findServer(const QString &strName);

extern QHash<unsigned int, MServer *> serverList;
extern MServer *uplinkServer;
extern MServer *virtualServer;

#endif /* __MS_SERVERS_H */
